﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public enum Direction
{
    RIGHT,
    LEFT
}

public enum CharacterType
{
    PLAYER,
    ENEMY1,
    ENEMY2
}

public class CharacterScript : MonoBehaviour {

    public CharacterType playerControlled;   //If this is true, the character is controlled by the player. Otherwise, we have to figure out AI.

    //Character Resource Variables
    public float characterHealth;
    public float maxHealth;
    public Image healthBarFull;
    public Image healthBarHurt;
    public GameObject[] drops;
    public float dropChance;

    //Movement Variables
    public float verticalWalkSpeed;
    public float horizontalWalkSpeed;
    public float flipSpeed;
    public GameObject characterSprite;

    //Combat Variables
    public float attackStartUpTime;
    public float attackActiveTime;
    public float attackRecoveryTime;
    public GameObject attackHitbox;
    public float stunTime;
    public GameObject hitParticle;
    public GameObject deathParticle;

    //Sound Assets
    public AudioSource source;
    public AudioClip attack;
    public AudioClip hit;
    public AudioClip healthPickup;
    public AudioClip deathTune;

    Animator anim;
    Direction characterDirection;
    Rigidbody rigid;
    Vector3 characterVelocity;
    bool canAttack = true;
    public bool canMove = true;
    GameObject target;
    GameManager gameManager;
    public Text name;

    // Use this for initialization
    void Start()
    {
        if (transform.tag == "Player")
        {
            name.text = GameObject.FindGameObjectWithTag("Menu").GetComponent<StartMenuScript>().playerName;
        }
        else
        {
            if (name != null)
            {
                name.text = "Player";
            }
        }

        source = GetComponent<AudioSource>();
        gameManager = GameObject.FindGameObjectWithTag("MainCamera").GetComponent<GameManager>();
        anim = characterSprite.GetComponent<Animator>();
        rigid = GetComponent<Rigidbody>();
        target = GameObject.FindGameObjectWithTag("Player");    //For use with AI
    }

    void Update()
    {
        if (Input.GetKeyDown(KeyCode.K) || Input.GetButtonDown("Fire1"))
        {
            if (canAttack == true && characterHealth > 0)
            {
                StartCoroutine(Attack());
            }
        }

        healthBarFull.fillAmount = characterHealth / maxHealth;
        if (characterHealth > maxHealth)
        {
            characterHealth = maxHealth;
        }
    }

	void FixedUpdate ()
    {
        if (canMove && playerControlled == CharacterType.PLAYER)
        {
            //Player Movement
            if (Input.GetAxis("Vertical") != 0)
            {
                characterVelocity.z = verticalWalkSpeed * Input.GetAxis("Vertical");
            }

            if (Input.GetAxis("Horizontal") != 0)
            {
                if (characterDirection == Direction.LEFT && Input.GetAxis("Horizontal") > 0)
                {
                    StartCoroutine(FlipDirection(Direction.RIGHT));
                }
                else if (characterDirection == Direction.RIGHT && Input.GetAxis("Horizontal") < 0)
                {
                    StartCoroutine(FlipDirection(Direction.LEFT));
                }

                characterVelocity.x = horizontalWalkSpeed * Input.GetAxis("Horizontal");
            }
            else
            {
                characterVelocity.x = 0;
            }

            characterVelocity.y = rigid.velocity.y;

            rigid.velocity = characterVelocity;
        }
        else if (canMove && playerControlled == CharacterType.ENEMY1)
        {
            //Enemy AI 1
            if (target.transform.position.z < transform.position.z)
            {
                characterVelocity.z = -verticalWalkSpeed;
            }
            else if (target.transform.position.z > transform.position.z)
            {
                characterVelocity.z = +verticalWalkSpeed;
            }
            if (target.transform.position.x < transform.position.x)
            {
                characterVelocity.x = -horizontalWalkSpeed;
                if (characterDirection == Direction.RIGHT)
                {
                    StartCoroutine(FlipDirection(Direction.LEFT));
                }
            }
            else if (target.transform.position.x > transform.position.x)
            {
                characterVelocity.x = horizontalWalkSpeed;
                if (characterDirection == Direction.LEFT)
                {
                    StartCoroutine(FlipDirection(Direction.RIGHT));
                }
            }

            rigid.velocity = characterVelocity;
        }

        if (anim != null && transform.tag == "Player")
        {
            anim.SetFloat("WalkSpeed", Mathf.Abs(characterVelocity.x));
        }
	}

    IEnumerator FlipDirection(Direction targetDirection)
    {
        if (characterDirection == Direction.RIGHT)
        {
            characterDirection = Direction.LEFT;
            //TODO: Rotate the sprite counterclockwise over a certain amount of time
            while (characterSprite.transform.rotation.eulerAngles.y < 180)
            {
                characterSprite.transform.Rotate(0, flipSpeed, 0);
                yield return new WaitForFixedUpdate();
            }
            if (characterSprite.transform.rotation.eulerAngles.y > 180)
            {
                characterSprite.transform.rotation.SetEulerAngles(0, 180, 0);
            }
        }
        else if (characterDirection == Direction.LEFT)
        {
            characterDirection = Direction.RIGHT;
            //TODO: Rotate the sprite clockwise over a certain amount of time
            while (characterSprite.transform.rotation.eulerAngles.y > 0)
            {
                characterSprite.transform.Rotate(0, -flipSpeed, 0);
                yield return new WaitForFixedUpdate();
            }
            if (characterSprite.transform.rotation.eulerAngles.y < 0)
            {
                characterSprite.transform.rotation.SetEulerAngles(0, 0, 0);
            }
        }
        yield return null;
    }

    IEnumerator Attack()
    {
        if (canAttack)
        {
            if (anim != null)
            {
                anim.SetBool("IsAttacking", true);
            }
            canAttack = false;
            canMove = false;
            source.PlayOneShot(attack);
            characterVelocity = Vector3.zero;
            yield return new WaitForSeconds(attackStartUpTime);

            attackHitbox.SetActive(true);
            yield return new WaitForSeconds(attackActiveTime);
            attackHitbox.SetActive(false);

            yield return new WaitForSeconds(attackRecoveryTime);
            if (anim != null)
            {
                anim.SetBool("IsAttacking", false);
            }
            canAttack = true;
            canMove = true;
        }
    }

    public void Damage(int damage)
    {
        StartCoroutine(TakeDamage(damage));
    }

    public IEnumerator TakeDamage(int damage)
    {
        if (characterHealth > 0)
        {
            //Reduce Health
            characterHealth -= damage;
            Debug.Log(transform.name + " has taken " + damage.ToString() + " damage!");
            Instantiate(hitParticle, transform.position, Quaternion.identity);
            source.PlayOneShot(hit);
            //Check Death
            if (characterHealth < 1)
            {
                StartCoroutine(Death());
                yield return null;
            }

            //TODO: Flash Red and disable movement/attack for a bit.
            canMove = false;
            canAttack = false;

            if (characterDirection == Direction.LEFT)
            {
                rigid.velocity = new Vector3(rigid.velocity.x + 2, rigid.velocity.y + 2, rigid.velocity.z);
            }
            else if (characterDirection == Direction.RIGHT)
            {
                rigid.velocity = new Vector3(rigid.velocity.x - 2, rigid.velocity.y + 2, rigid.velocity.z);
            }

            yield return new WaitForSeconds(1);

            canMove = true;
            canAttack = true;
        }
    }

    public IEnumerator Death()
    {
        //Disable the player
        canAttack = false;
        canMove = false;
        StopCoroutine(Attack());
        rigid.velocity = Vector3.zero;

        Debug.Log(transform.name + " has been killed!");
        if (transform.tag == "Player")
        {
            gameManager.camera.ToggleScreenFade();
            source.Stop();
            source.PlayOneShot(deathTune);
            if (gameManager.roomsCleared > PlayerPrefs.GetInt("HighScore", 0))
            {
                PlayerPrefs.SetInt("HighScore", gameManager.roomsCleared);
                PlayerPrefs.SetString("HighScoreName", name.text);
            }
            Destroy(GameObject.FindGameObjectWithTag("Menu").gameObject);
            yield return new WaitForSeconds(3);
            Application.LoadLevel(0);
        }
        else
        {
            if (Random.Range(1, 100) < dropChance)
            {
                Instantiate(drops[Random.Range(0, drops.Length)], transform.position, Quaternion.identity);
            }
            Instantiate(deathParticle, transform.position, Quaternion.identity);
            Destroy(this.gameObject);
        }
        yield return null;
    }
}
